"""Computational processing related to image processing."""

from typing import Tuple


def _calc_internally(p1: float, p2: float, ratio: Tuple[float, float]) -> float:
    """Calculate the point that divides internally.

    Args:
        p1 (float): point 1
        p2 (float): point 2
        ratio (float): ratio of division

    Returns:
        float: divided point
    """
    num_ratio = ratio[0] / (ratio[0] + ratio[1])
    assert 0 <= num_ratio <= 1
    if p1 < p2:
        dist = p2 - p1
        return p1 + dist * num_ratio
    else:
        dist = p1 - p2
        return p1 - dist * num_ratio


def divide_internally(p1: Tuple[int, int], p2: Tuple[int, int], ratio: Tuple[float, float]) -> Tuple[int, int]:
    """Obtain internally division point.

    obtain the point that divides internally

    Args:
        p1 (Tuple[int, int]): point 1
        p2 (Tuple[int, int]): point 2
        ratio (float): ratio of division

    Returns:
        Tuple[int, int]: divided point

    """
    x = _calc_internally(p1[0], p2[0], ratio)
    y = _calc_internally(p1[1], p2[1], ratio)

    return (int(x), int(y))


def _calc_externally(p1: float, p2: float, ratio: Tuple[float, float]) -> float:
    """Calculate the point that divides externally.

    Args:
        p1 (float): point 1
        p2 (float): point 2
        ratio (float): rate of division

    Returns:
        float: divided point
    """
    if ratio[0] >= ratio[1]:
        num_ratio = ratio[0] / (ratio[0] - ratio[1])
        assert num_ratio >= 1
        if p1 < p2:
            dist = p2 - p1
            return p1 + dist * num_ratio
        else:
            dist = p1 - p2
            return p2 + dist * num_ratio
    else:
        num_ratio = ratio[1] / (ratio[1] - ratio[0])
        assert num_ratio >= 1
        if p1 < p2:
            dist = p2 - p1
            return p1 + dist * num_ratio
        else:
            dist = p1 - p2
            return p2 + dist * num_ratio


def divide_externally(p1: Tuple[int, int], p2: Tuple[int, int], ratio: Tuple[float, float]) -> Tuple[int, int]:
    """Obtain externally division point.

    obtain the point that divides internally

    Args:
        p1 (Tuple[int, int]): point 1
        p2 (Tuple[int, int]): point 2
        ratio (float): ratio of division

    Returns:
        Tuple[int, int]: divided point

    """
    x = _calc_externally(p1[0], p2[0], ratio)
    y = _calc_externally(p1[1], p2[1], ratio)

    return (int(x), int(y))



"""Processing for the image."""

from typing import Tuple

import cv2
import numpy as np


def resize(img: np.ndarray, output_size: Tuple[int, int], padding: bool = True) -> np.ndarray:
    """Resize the image.

    resize the image with padding or crop

    Args:
        img (numpy.ndarray): OpenCV image
        output_size (Tuple[int, int]): size of the output image
        padding (bool): whether to perform padding or not

    Returns:
        numpy.ndarray: OpenCV image

    """
    x = img.shape[1]
    y = img.shape[0]
    out_x, out_y = output_size
    in_rate = x / y
    out_rate = out_x / out_y
    if padding:
        if in_rate == out_rate:
            resized = np.copy(img)
        elif in_rate >= out_rate:
            resized_y = int(x / out_rate)
            start = (resized_y - y) // 2
            end = (resized_y + y) // 2
            resized = cv2.resize(np.zeros((1, 1, 3), np.uint8), (x, resized_y))
            resized[start:end, :] = img
        else:
            resize_x = int(y * out_rate)
            start = (resize_x - x) // 2
            end = (resize_x + x) // 2
            resized = cv2.resize(np.zeros((1, 1, 3), np.uint8), (resize_x, y))
            resized[:, start:end] = img
    else:
        if in_rate == out_rate:
            resized = np.copy(img)
        elif in_rate >= out_rate:
            resize_x = int(y * out_rate)
            d = x - resize_x
            gap_x = d // 2
            resized = img[:, gap_x : gap_x + resize_x]
        else:
            resize_y = int(x / out_rate)
            d = y - resize_y
            gap_y = d // 2
            resized = img[gap_y : gap_y + resize_y, :]
    resized = cv2.resize(resized, dsize=output_size)

    return resized



"""A package to help with image processing."""

__version__ = "0.1.0"

from .calc import divide_externally, divide_internally
from .conv import cv2pil, pil2cv
from .proc import resize

__all__ = ["divide_internally", "divide_externally", "cv2pil", "pil2cv", "resize"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['imagica']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=9.0.1,<10.0.0', 'opencv-python>=4.5.5,<5.0.0']

setup_kwargs = {
    'name': 'imagica',
    'version': '0.1.0',
    'description': 'A package to help with image processing',
    'long_description': '# imagica\nA package to help with image processing for Python\n',
    'author': 'MagicalLiebe',
    'author_email': 'magical.liebe@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/MagicalLiebe',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""Convert the image format."""

import numpy as np
from PIL import Image


def cv2pil(img: np.ndarray) -> Image.Image:
    """Convert OpenCV->PIL.

    convert the format from OpenCV to PIL

    Args:
        img (numpy.ndarray): OpenCV image

    Returns:
        PIL.Image.Image: PIL image

    """
    converted = img.copy()
    if converted.ndim == 2:  # grayscale
        pass
    elif converted.shape[2] == 3:  # RGB
        converted = converted[:, :, ::-1]
    elif converted.shape[2] == 4:  # RGBA
        converted = converted[:, :, [2, 1, 0, 3]]
    converted = Image.fromarray(converted)
    return converted


def pil2cv(img: Image.Image) -> np.ndarray:
    """Convert PIL->OpenCV.

    convert the format from PIL to OpenCV

    Args:
        img (PIL.Image.Imag): PIL image

    Returns:
        numpy.ndarray: OpenCV image

    """
    converted = np.array(img, dtype=np.uint8)
    if converted.ndim == 2:  # grayscale
        pass
    elif converted.shape[2] == 3:  # RGB
        converted = converted[:, :, ::-1]
    elif converted.shape[2] == 4:  # RGBA
        converted = converted[:, :, [2, 1, 0, 3]]
    return converted
